@@ -1,16 +1,15 @@
 """iZone device discovery."""
 
 import asyncio
-from async_timeout import timeout
 import logging
 from abc import abstractmethod, ABC
 from asyncio import (AbstractEventLoop, Condition, DatagramProtocol,
-                     DatagramTransport, Future, Task, )
+                     DatagramTransport, Future, Task, CancelledError, )
 from logging import Logger
 from typing import Dict, List, Set, Optional
 
+from async_timeout import timeout
 from aiohttp import ClientSession
-
 import netifaces
 
 from .controller import Controller
@@ -154,7 +153,7 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
 
         self._transport = None  # type: Optional[DatagramTransport]
 
-        self._scan_condition = Condition(loop=self.loop)  # type: Condition
+        self._scan_condition = Condition()  # type: Condition
 
         self._tasks = []  # type: List[Future]
 
@@ -167,8 +166,11 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
         await self.close()
 
     def _task_done_callback(self, task):
-        if task.exception():
-            _LOG.exception("Uncaught exception", exc_info=task.exception())
+        try:
+            if task.exception():
+                _LOG.exception("Uncaught exception", exc_info=task.exception())
+        except CancelledError:
+            pass
         self._tasks.remove(task)
 
     # managing the task list.
@@ -209,7 +211,7 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
             "Connection to controller lost: id=%s ip=%s",
             ctrl.device_uid, ctrl.device_ip)
         self._disconnected.add(ctrl.device_uid)
-        self.loop.create_task(self._rescan())
+        self.create_task(self._rescan())
         for listener in self._listeners:
             with LogExceptions("controller_disconnected"):
                 listener.controller_disconnected(ctrl, ex)
@@ -257,12 +259,12 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
         self.create_task(self._scan_loop())
 
     def _get_broadcasts(self):
-        for ifAddr in map(netifaces.ifaddresses, netifaces.interfaces()):
-            inetAddrs = ifAddr.get(netifaces.AF_INET)
-            if not inetAddrs:
+        for ifaddr in map(netifaces.ifaddresses, netifaces.interfaces()):
+            inetaddrs = ifaddr.get(netifaces.AF_INET)
+            if not inetaddrs:
                 continue
-            for inetAddr in inetAddrs:
-                broadcast = inetAddr.get('broadcast')
+            for inetaddr in inetaddrs:
+                broadcast = inetaddr.get('broadcast')
                 if broadcast:
                     yield broadcast
 
@@ -306,11 +308,12 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
             await self._close_task
             return
         _LOG.info("Close called on discovery service.")
-        self._close_task = Task.current_task(self.loop)
+        self._close_task = asyncio.current_task(loop=self.loop)
         if self._transport:
             self._transport.close()
 
-        await self._rescan()
+        for i in self._tasks:
+            i.cancel()
 
         if self._own_session and self.session:
             await self.session.close()
@@ -362,14 +365,14 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
             ctrl = self._find_by_addr(addr)
             if not ctrl:
                 return
-            self.create_task(self._wrap_update(ctrl._refresh_system()))  # pylint: disable=protected-access  # noqa: E501
+            return self.create_task(self._wrap_update(ctrl._refresh_system()))  # pylint: disable=protected-access  # noqa: E501
         elif data == CHANGED_ZONES:
             ctrl = self._find_by_addr(addr)
             if not ctrl:
                 return
-            self.create_task(self._wrap_update(ctrl._refresh_zones()))  # pylint: disable=protected-access  # noqa: E501
+            return self.create_task(self._wrap_update(ctrl._refresh_zones()))  # pylint: disable=protected-access  # noqa: E501
         else:
-            self._discovery_recieved(data)
+            return self._discovery_recieved(data)
 
     def _discovery_recieved(self, data):
         message = data.decode().split(',')
@@ -401,7 +404,7 @@ class DiscoveryService(AbstractDiscoveryService, DatagramProtocol, Listener):
                 self._controllers[device_uid] = controller
                 self.controller_discovered(controller)
 
-            self.create_task(initialize_controller())
+            return self.create_task(initialize_controller())
         else:
             controller = self._controllers[device_uid]
             controller._refresh_address(device_ip)  # pylint: disable=protected-access  # noqa: E501
